@@ -2,6 +2,7 @@ use chrono::{DateTime, FixedOffset};
 use lazy_static::lazy_static;
 use regex::Regex;
 use std::borrow::Cow;
+use unicode_width::UnicodeWidthStr;
 
 use crate::ansi::measure_text_width;
 use crate::color;
@@ -76,13 +77,9 @@ impl<'a> StateMachine<'a> {
 
                 // Emit syntax-highlighted code
                 if matches!(self.state, State::Unknown) {
-                    if let Some(lang) = utils::process::git_blame_filename_extension()
-                        .as_ref()
-                        .or(self.config.default_language.as_ref())
-                    {
-                        self.painter.set_syntax(Some(lang));
-                        self.painter.set_highlighter();
-                    }
+                    self.painter
+                        .set_syntax(utils::process::git_blame_filename().as_deref());
+                    self.painter.set_highlighter();
                 }
                 self.state = State::Blame(key);
                 self.painter.syntax_highlight_and_paint_line(
@@ -281,9 +278,12 @@ pub fn format_blame_metadata(
             _ => unreachable!("Unexpected `git blame` input"),
         };
         if let Some(field) = field {
+            // Unicode modifier should not be counted as character to allow a consistent padding
+            let unicode_modifier_width =
+                field.as_ref().chars().count() - UnicodeWidthStr::width(field.as_ref());
             s.push_str(&format::pad(
                 &field,
-                width,
+                width + unicode_modifier_width,
                 alignment_spec,
                 placeholder.precision,
             ))
@@ -357,7 +357,7 @@ pub fn parse_blame_line_numbers(arg: &str) -> BlameLineNumbers {
 
     match f.fmt_type.as_str() {
         t if t.is_empty() || t == "every" => BlameLineNumbers::On(set_defaults(f.into_simple())),
-        t if t == "block" => BlameLineNumbers::PerBlock(set_defaults(f.into_simple())),
+        "block" => BlameLineNumbers::PerBlock(set_defaults(f.into_simple())),
         every_n if every_n.starts_with("every-") => {
             let n = every_n["every-".len()..]
                 .parse::<usize>()
@@ -411,10 +411,7 @@ mod tests {
 
     #[test]
     fn test_format_blame_metadata_with_default_timestamp_output_format() {
-        let format_data = format::FormatStringPlaceholderData {
-            placeholder: Some(Placeholder::Str("timestamp")),
-            ..Default::default()
-        };
+        let format_data = make_format_data_with_placeholder("timestamp");
         let blame = make_blame_line_with_time("1996-12-19T16:39:57-08:00");
         let config = integration_test_utils::make_config_from_args(&[]);
         let regex = Regex::new(r"^\d+ years ago$").unwrap();
@@ -424,10 +421,7 @@ mod tests {
 
     #[test]
     fn test_format_blame_metadata_with_custom_timestamp_output_format() {
-        let format_data = format::FormatStringPlaceholderData {
-            placeholder: Some(Placeholder::Str("timestamp")),
-            ..Default::default()
-        };
+        let format_data = make_format_data_with_placeholder("timestamp");
         let blame = make_blame_line_with_time("1996-12-19T16:39:57-08:00");
         let config = integration_test_utils::make_config_from_args(&[
             "--blame-timestamp-output-format=%Y-%m-%d %H:%M",
@@ -436,6 +430,26 @@ mod tests {
         assert_eq!(result.trim(), "1996-12-19 16:39");
     }
 
+    #[test]
+    fn test_format_blame_metadata_with_accent_in_name() {
+        let config = integration_test_utils::make_config_from_args(&[]);
+
+        let count_trailing_spaces = |s: String| s.chars().rev().filter(|&c| c == ' ').count();
+
+        let format_data1 = make_format_data_with_placeholder("author");
+        let blame1 = make_blame_line_with_author("E\u{301}dith Piaf");
+        let result1 = format_blame_metadata(&[format_data1], &blame1, &config);
+
+        let format_data2 = make_format_data_with_placeholder("author");
+        let blame2 = make_blame_line_with_author("Edith Piaf");
+        let result2 = format_blame_metadata(&[format_data2], &blame2, &config);
+
+        assert_eq!(
+            count_trailing_spaces(result1),
+            count_trailing_spaces(result2)
+        );
+    }
+
     #[test]
     fn test_color_assignment() {
         let mut writer = Cursor::new(vec![0; 512]);
@@ -536,4 +550,21 @@ mod tests {
             code: "",
         }
     }
+
+    fn make_format_data_with_placeholder(placeholder: &str) -> format::FormatStringPlaceholderData {
+        format::FormatStringPlaceholderData {
+            placeholder: Some(Placeholder::Str(placeholder)),
+            ..Default::default()
+        }
+    }
+
+    fn make_blame_line_with_author(author: &str) -> BlameLine {
+        BlameLine {
+            commit: "",
+            author,
+            time: chrono::DateTime::default(),
+            line_number: 0,
+            code: "",
+        }
+    }
 }
