@@ -6,7 +6,8 @@ use serde::Deserialize;
 
 use crate::ansi;
 use crate::config::{
-    delta_unreachable, GrepType, HunkHeaderIncludeFilePath, HunkHeaderIncludeLineNumber,
+    delta_unreachable, GrepType, HunkHeaderIncludeCodeFragment, HunkHeaderIncludeFilePath,
+    HunkHeaderIncludeLineNumber,
 };
 use crate::delta::{State, StateMachine};
 use crate::handlers::{self, ripgrep_json};
@@ -26,6 +27,24 @@ pub struct GrepLine<'b> {
     pub submatches: Option<Vec<(usize, usize)>>,
 }
 
+impl<'b> GrepLine<'b> {
+    fn expand_tabs(&mut self, tab_cfg: &tabs::TabCfg) {
+        let old_len = self.code.len();
+        self.code = tabs::expand(&self.code, tab_cfg).into();
+        let shift = self.code.len().saturating_sub(old_len);
+        // HACK: it is not necessarily the case that all submatch coordinates
+        // should be shifted in this way. It should be true in a common case of:
+        // (a) the only tabs were at the beginning of the line, and (b) the user
+        // was not searching for tabs.
+        self.submatches = self.submatches.as_ref().map(|submatches| {
+            submatches
+                .iter()
+                .map(|(a, b)| (a + shift, b + shift))
+                .collect()
+        });
+    }
+}
+
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Deserialize)]
 #[serde(rename_all = "lowercase")]
 pub enum LineType {
@@ -72,55 +91,60 @@ impl<'a> StateMachine<'a> {
             State::Unknown => (None, None, &None, true),
             _ => (None, None, &None, false),
         };
-
-        let mut handled_line = false;
-        if try_parse {
-            let line = self.line.clone(); // TODO: avoid clone
-            if let Some(grep_line) = parse_grep_line(&line) {
-                if matches!(grep_line.line_type, LineType::Ignore) {
-                    handled_line = true;
-                    return Ok(handled_line);
-                }
-                let first_path = previous_path.is_none();
-                let new_path = first_path || previous_path.as_deref() != Some(&grep_line.path);
-                // Emit a '--' section separator when output contains context lines (i.e. *grep option -A, -B, -C is in effect).
-                let new_section = !new_path
-                    && (previous_line_type == Some(&LineType::Context)
-                        || grep_line.line_type == LineType::Context)
-                    && previous_line < &grep_line.line_number.as_ref().map(|n| n - 1);
-                self.state = State::Grep(
-                    self.config
-                        .grep_output_type
-                        .clone()
-                        .unwrap_or_else(|| grep_line.grep_type.clone()),
-                    grep_line.line_type,
-                    grep_line.path.to_string(),
-                    grep_line.line_number,
-                );
-                if new_path {
-                    if let Some(lang) = handlers::diff_header::get_extension(&grep_line.path)
-                        .or(self.config.default_language.as_deref())
-                    {
-                        self.painter.set_syntax(Some(lang));
-                        self.painter.set_highlighter();
-                    }
-                }
-                match &self.state {
-                    State::Grep(GrepType::Ripgrep, _, _, _) => self.emit_ripgrep_format_grep_line(
-                        grep_line,
-                        new_path,
-                        first_path,
-                        new_section,
-                    ),
-                    State::Grep(GrepType::Classic, _, _, _) => {
-                        self.emit_classic_format_grep_line(grep_line)
-                    }
-                    _ => delta_unreachable("Impossible state while handling grep line."),
-                }?;
-                handled_line = true
-            }
+        if !try_parse {
+            return Ok(false);
         }
-        Ok(handled_line)
+
+        // Try parse_raw_grep_line on raw_line, and fall back to parse_grep_line
+        let raw_line = self.raw_line.clone(); // TODO: avoid clone
+        let line;
+        let grep_line = if let Some(grep_line) = parse_raw_grep_line(&raw_line) {
+            grep_line
+        } else {
+            line = self.line.clone(); // TODO: avoid clone
+            if let Some(grep_line) = parse_grep_line(&line) {
+                grep_line
+            } else {
+                return Ok(false);
+            }
+        };
+
+        if matches!(grep_line.line_type, LineType::Ignore) {
+            return Ok(true);
+        }
+        let first_path = previous_path.is_none();
+        let new_path = first_path || previous_path.as_deref() != Some(&grep_line.path);
+        // Emit a '--' section separator when output contains context lines (i.e. *grep option -A, -B, -C is in effect).
+        let new_section = !new_path
+            && (previous_line_type == Some(&LineType::Context)
+                || grep_line.line_type == LineType::Context)
+            && previous_line < &grep_line.line_number.as_ref().map(|n| n - 1);
+        self.state = State::Grep(
+            self.config
+                .grep_output_type
+                .clone()
+                .unwrap_or_else(|| grep_line.grep_type.clone()),
+            grep_line.line_type,
+            grep_line.path.to_string(),
+            grep_line.line_number,
+        );
+        if new_section {
+            self.painter.set_highlighter()
+        }
+        if new_path {
+            self.painter.set_syntax(Some(grep_line.path.as_ref()));
+            self.painter.set_highlighter();
+        }
+        match &self.state {
+            State::Grep(GrepType::Ripgrep, _, _, _) => {
+                self.emit_ripgrep_format_grep_line(grep_line, new_path, first_path, new_section)
+            }
+            State::Grep(GrepType::Classic, _, _, _) => {
+                self.emit_classic_format_grep_line(grep_line)
+            }
+            _ => delta_unreachable("Impossible state while handling grep line."),
+        }?;
+        Ok(true)
     }
 
     // Emulate ripgrep output: each section of hits from the same path has a header line,
@@ -150,6 +174,7 @@ impl<'a> StateMachine<'a> {
                 &HunkHeaderIncludeFilePath::Yes,
                 &HunkHeaderIncludeLineNumber::No,
                 &HunkHeaderIncludeHunkLabel::Yes,
+                &HunkHeaderIncludeCodeFragment::Yes,
                 "",
                 self.config,
             )?
@@ -159,17 +184,17 @@ impl<'a> StateMachine<'a> {
         }
         // Emit the actual grep hit line
         let code_style_sections = match (&grep_line.line_type, &grep_line.submatches) {
-            (LineType::Match, Some(submatches)) => {
+            (LineType::Match, Some(_)) => {
                 // We expand tabs at this late stage because
                 // the tabs are escaped in the JSON, so
                 // expansion must come after JSON parsing.
                 // (At the time of writing, we are in this
                 // arm iff we are handling `ripgrep --json`
                 // output.)
-                grep_line.code = tabs::expand(&grep_line.code, &self.config.tab_cfg).into();
+                grep_line.expand_tabs(&self.config.tab_cfg);
                 make_style_sections(
                     &grep_line.code,
-                    submatches,
+                    &grep_line.submatches.unwrap(),
                     self.config.grep_match_word_style,
                     self.config.grep_match_line_style,
                 )
@@ -186,7 +211,8 @@ impl<'a> StateMachine<'a> {
                     &self.raw_line,
                     self.config.grep_match_word_style,
                     self.config.grep_match_line_style,
-                    &grep_line,
+                    &grep_line.path,
+                    grep_line.line_number,
                 )
                 .unwrap_or(StyleSectionSpecifier::Style(
                     self.config.grep_match_line_style,
@@ -211,6 +237,7 @@ impl<'a> StateMachine<'a> {
                 &HunkHeaderIncludeLineNumber::No
             },
             &HunkHeaderIncludeHunkLabel::No,
+            &HunkHeaderIncludeCodeFragment::Yes,
             grep_line.line_type.file_path_separator(),
             self.config,
         )
@@ -236,6 +263,7 @@ impl<'a> StateMachine<'a> {
                     &self.config.hunk_header_style_include_file_path,
                     &self.config.hunk_header_style_include_line_number,
                     &HunkHeaderIncludeHunkLabel::Yes,
+                    &HunkHeaderIncludeCodeFragment::Yes,
                     grep_line.line_type.file_path_separator(),
                     self.config,
                 )?
@@ -297,17 +325,17 @@ impl<'a> StateMachine<'a> {
 
     fn _emit_classic_format_code(&mut self, mut grep_line: GrepLine) -> std::io::Result<()> {
         let code_style_sections = match (&grep_line.line_type, &grep_line.submatches) {
-            (LineType::Match, Some(submatches)) => {
+            (LineType::Match, Some(_)) => {
                 // We expand tabs at this late stage because
                 // the tabs are escaped in the JSON, so
                 // expansion must come after JSON parsing.
                 // (At the time of writing, we are in this
                 // arm iff we are handling `ripgrep --json`
                 // output.)
-                grep_line.code = tabs::expand(&grep_line.code, &self.config.tab_cfg).into();
+                grep_line.expand_tabs(&self.config.tab_cfg);
                 make_style_sections(
                     &grep_line.code,
-                    submatches,
+                    &grep_line.submatches.unwrap(),
                     self.config.grep_match_word_style,
                     self.config.grep_match_line_style,
                 )
@@ -324,7 +352,8 @@ impl<'a> StateMachine<'a> {
                     &self.raw_line,
                     self.config.grep_match_word_style,
                     self.config.grep_match_line_style,
-                    &grep_line,
+                    &grep_line.path,
+                    grep_line.line_number,
                 )
                 .unwrap_or(StyleSectionSpecifier::Style(
                     self.config.grep_match_line_style,
@@ -369,13 +398,14 @@ fn get_code_style_sections<'b>(
     raw_line: &'b str,
     match_style: Style,
     non_match_style: Style,
-    grep: &GrepLine,
+    path: &str,
+    line_number: Option<usize>,
 ) -> Option<StyleSectionSpecifier<'b>> {
     if let Some(prefix_end) = ansi::ansi_preserving_index(
         raw_line,
-        match grep.line_number {
-            Some(n) => format!("{}:{}:", grep.path, n).len() - 1,
-            None => grep.path.len(),
+        match line_number {
+            Some(n) => format!("{}:{}:", path, n).len() - 1,
+            None => path.len(),
         },
     ) {
         let match_style_sections = ansi::parse_style_sections(&raw_line[(prefix_end + 1)..])
@@ -439,12 +469,18 @@ fn make_output_config() -> GrepOutputConfig {
 }
 
 enum GrepLineRegex {
+    WithColor,
     WithFileExtensionAndLineNumber,
     WithFileExtension,
     WithFileExtensionNoSpaces,
     WithoutSeparatorCharacters,
 }
 
+lazy_static! {
+    static ref GREP_LINE_REGEX_ASSUMING_COLOR: Regex =
+        make_grep_line_regex(GrepLineRegex::WithColor);
+}
+
 lazy_static! {
     static ref GREP_LINE_REGEX_ASSUMING_FILE_EXTENSION_AND_LINE_NUMBER: Regex =
         make_grep_line_regex(GrepLineRegex::WithFileExtensionAndLineNumber);
@@ -491,6 +527,15 @@ fn make_grep_line_regex(regex_variant: GrepLineRegex) -> Regex {
     // Make-7-file-7-xxx
 
     let file_path = match regex_variant {
+        GrepLineRegex::WithColor => {
+            r"
+            \x1b\[35m               # starts with ANSI color sequence
+        (                        # 1. file name
+            [^\x1b]*                # anything
+        )
+            \x1b\[m                 # ends with ANSI color reset
+        "
+        }
         GrepLineRegex::WithFileExtensionAndLineNumber | GrepLineRegex::WithFileExtension => {
             r"
         (                        # 1. file name (colons not allowed)
@@ -520,6 +565,51 @@ fn make_grep_line_regex(regex_variant: GrepLineRegex) -> Regex {
     };
 
     let separator = match regex_variant {
+        GrepLineRegex::WithColor => {
+            r#"
+            \x1b\[36m        # starts with ANSI color sequence for separator
+    (?:
+        (
+            :                # 2. match marker
+            \x1b\[m          # ANSI color reset
+            (?:              # optional: line number followed by second match marker
+                \x1b\[32m        # ANSI color sequence for line number
+                ([0-9]+)         # 3. line number
+                \x1b\[m          # ANSI color reset
+                \x1b\[36m        # ANSI color sequence for separator
+                :                # second match marker
+                \x1b\[m          # ANSI color reset
+            )?
+        )
+        |
+        (
+            -                # 4. nomatch marker
+            \x1b\[m          # ANSI color reset
+            (?:              # optional: line number followed by second nomatch marker
+                \x1b\[32m        # ANSI color sequence for line number
+                ([0-9]+)         # 5. line number
+                \x1b\[m          # ANSI color reset
+                \x1b\[36m        # ANSI color sequence for separator
+                -                # second nomatch marker
+                \x1b\[m          # ANSI color reset
+            )?
+        )
+        |
+        (
+            =                # 6. context header marker
+            \x1b\[m          # ANSI color reset
+            (?:              # optional: line number followed by second context header marker
+                \x1b\[32m        # ANSI color sequence for line number
+                ([0-9]+)         # 7. line number
+                \x1b\[m          # ANSI color reset
+                \x1b\[36m        # ANSI color sequence for separator
+                =                # second context header marker
+                \x1b\[m          # ANSI color reset
+            )?
+        )
+    )
+            "#
+        }
         GrepLineRegex::WithFileExtensionAndLineNumber => {
             r#"
     (?:
@@ -592,6 +682,23 @@ pub fn parse_grep_line(line: &str) -> Option<GrepLine> {
     }
 }
 
+pub fn parse_raw_grep_line(raw_line: &str) -> Option<GrepLine> {
+    // Early exit if we don't have an escape sequence
+    if !raw_line.starts_with('\x1b') {
+        return None;
+    }
+    if !matches!(
+        &*process::calling_process(),
+        process::CallingProcess::GitGrep(_) | process::CallingProcess::OtherGrep
+    ) {
+        return None;
+    }
+    _parse_grep_line(&GREP_LINE_REGEX_ASSUMING_COLOR, raw_line).map(|mut grep_line| {
+        grep_line.code = ansi::strip_ansi_codes(&grep_line.code).into();
+        grep_line
+    })
+}
+
 pub fn _parse_grep_line<'b>(regex: &Regex, line: &'b str) -> Option<GrepLine<'b>> {
     let caps = regex.captures(line)?;
     let file = caps.get(1).unwrap().as_str().into();
@@ -624,7 +731,9 @@ pub fn _parse_grep_line<'b>(regex: &Regex, line: &'b str) -> Option<GrepLine<'b>
 
 #[cfg(test)]
 mod tests {
-    use crate::handlers::grep::{parse_grep_line, GrepLine, GrepType, LineType};
+    use crate::handlers::grep::{
+        parse_grep_line, parse_raw_grep_line, GrepLine, GrepType, LineType,
+    };
     use crate::utils::process::tests::FakeParentArgs;
 
     #[test]
@@ -837,6 +946,42 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_parse_grep_color() {
+        let fake_parent_grep_command =
+            "/usr/local/bin/git --doesnt-matter grep --nor-this nor_this -- nor_this";
+        let _args = FakeParentArgs::for_scope(fake_parent_grep_command);
+
+        let e = "\x1B";
+        assert_eq!(
+            parse_raw_grep_line(&format!(
+                r#"{e}[35msrc/zlib-ng/configure{e}[m{e}[36m:{e}[m    -a*=* | -{e}[1;32m-arch{e}[ms=*) ARCHS=$(echo $1 | sed 's/.*=//'); shift ;;"#
+            )),
+            Some(GrepLine {
+                grep_type: GrepType::Classic,
+                path: "src/zlib-ng/configure".into(),
+                line_number: None,
+                line_type: LineType::Match,
+                code: r#"    -a*=* | --archs=*) ARCHS=$(echo $1 | sed 's/.*=//'); shift ;;"#.into(),
+                submatches: None,
+            })
+        );
+
+        assert_eq!(
+            parse_raw_grep_line(&format!(
+                r#"{e}[35msrc/zlib-ng/configure{e}[m{e}[36m:{e}[m{e}[32m214{e}[m{e}[36m:{e}[m    -a*=* | -{e}[1;32m-arch{e}[ms=*) ARCHS=$(echo $1 | sed 's/.*=//'); shift ;;"#
+            )),
+            Some(GrepLine {
+                grep_type: GrepType::Classic,
+                path: "src/zlib-ng/configure".into(),
+                line_number: Some(214),
+                line_type: LineType::Match,
+                code: r#"    -a*=* | --archs=*) ARCHS=$(echo $1 | sed 's/.*=//'); shift ;;"#.into(),
+                submatches: None,
+            })
+        );
+    }
+
     #[test]
     fn test_parse_grep_no_match() {
         let fake_parent_grep_command =
@@ -1101,7 +1246,7 @@ mod tests {
         let grep = parse_grep_line(&stripped).unwrap();
 
         assert_eq!(
-            get_code_style_sections(&working_example, hit, miss, &grep),
+            get_code_style_sections(&working_example, hit, miss, &grep.path, grep.line_number),
             Some(StyleSectionSpecifier::StyleSections(vec![
                 (miss, "  - "),
                 (hit, "kind: Service"),
@@ -1114,7 +1259,13 @@ mod tests {
         let broken_grep = parse_grep_line(&broken_stripped).unwrap();
 
         assert_eq!(
-            get_code_style_sections(&broken_example, hit, miss, &broken_grep),
+            get_code_style_sections(
+                &broken_example,
+                hit,
+                miss,
+                &broken_grep.path,
+                broken_grep.line_number
+            ),
             Some(StyleSectionSpecifier::StyleSections(vec![(
                 hit,
                 "kind: Service"
@@ -1126,7 +1277,13 @@ mod tests {
         let plus_grep = parse_grep_line(&plus_stripped).unwrap();
 
         assert_eq!(
-            get_code_style_sections(&plus_example, hit, miss, &plus_grep),
+            get_code_style_sections(
+                &plus_example,
+                hit,
+                miss,
+                &plus_grep.path,
+                plus_grep.line_number
+            ),
             Some(StyleSectionSpecifier::StyleSections(vec![
                 (miss, " +        let (style, non_emph_style) = "),
                 (hit, "match"),
