@@ -70,8 +70,7 @@ pub enum StyleSectionSpecifier<'l> {
 
 impl<'p> Painter<'p> {
     pub fn new(writer: &'p mut dyn Write, config: &'p config::Config) -> Self {
-        let default_syntax = Self::get_syntax(&config.syntax_set, None);
-
+        let default_syntax = Self::get_syntax(&config.syntax_set, None, &config.default_language);
         let panel_width_fix = ansifill::UseFullPanelWidth::new(config);
 
         let line_numbers_data = if config.line_numbers {
@@ -103,19 +102,50 @@ impl<'p> Painter<'p> {
         }
     }
 
-    pub fn set_syntax(&mut self, extension: Option<&str>) {
-        self.syntax = Painter::get_syntax(&self.config.syntax_set, extension);
+    pub fn set_syntax(&mut self, filename: Option<&str>) {
+        self.syntax = Painter::get_syntax(
+            &self.config.syntax_set,
+            filename,
+            &self.config.default_language,
+        );
     }
 
-    fn get_syntax<'a>(syntax_set: &'a SyntaxSet, extension: Option<&str>) -> &'a SyntaxReference {
-        if let Some(extension) = extension {
-            if let Some(syntax) = syntax_set.find_syntax_by_extension(extension) {
-                return syntax;
+    fn get_syntax<'a>(
+        syntax_set: &'a SyntaxSet,
+        filename: Option<&str>,
+        fallback: &str,
+    ) -> &'a SyntaxReference {
+        if let Some(filename) = filename {
+            let path = std::path::Path::new(filename);
+            let file_name = path.file_name().and_then(|n| n.to_str()).unwrap_or("");
+            let extension = path.extension().and_then(|x| x.to_str()).unwrap_or("");
+
+            // Like syntect's `find_syntax_for_file`, without inspecting the file content, plus:
+            // If the file has NO extension then look up the whole filename as a
+            // syntax definition (if it is longer than 4 bytes).
+            // This means file formats like Makefile/Dockerfile/Rakefile etc. will get highlighted,
+            // but 1-4 short filenames will not -- even if they, as a whole, match an extension:
+            // 'rs' will not get highlighted, while 'x.rs' will.
+            if !extension.is_empty() || file_name.len() > 4 {
+                if let Some(syntax) = syntax_set
+                    .find_syntax_by_extension(file_name)
+                    .or_else(|| syntax_set.find_syntax_by_extension(extension))
+                {
+                    return syntax;
+                }
             }
         }
-        syntax_set
-            .find_syntax_by_extension("txt")
-            .unwrap_or_else(|| delta_unreachable("Failed to find any language syntax definitions."))
+
+        // Nothing found, try the user provided fallback, or the internal fallback.
+        if let Some(syntax) = syntax_set.find_syntax_for_file(fallback).unwrap_or(None) {
+            syntax
+        } else {
+            syntax_set
+                .find_syntax_by_extension(config::SYNTAX_FALLBACK_LANG)
+                .unwrap_or_else(|| {
+                    delta_unreachable("Failed to find any language syntax definitions.")
+                })
+        }
     }
 
     pub fn set_highlighter(&mut self) {
@@ -666,11 +696,40 @@ pub fn get_syntax_style_sections_for_lines<'a>(
     ) {
         (Some(highlighter), true) => {
             for (line, _) in lines.iter() {
-                line_sections.push(
-                    highlighter
-                        .highlight_line(line, &config.syntax_set)
-                        .unwrap(),
-                );
+                // Fast but simple length comparison. Overcounts non-printable ansi
+                // characters or wider UTF-8, but `truncate_str_short` in the
+                // else branch corrects that.
+                if line.len() < config.max_syntax_length || config.max_syntax_length == 0 {
+                    line_sections.push(
+                        highlighter
+                            .highlight_line(line, &config.syntax_set)
+                            .unwrap(),
+                    );
+                } else {
+                    let line_syntax = ansi::truncate_str_short(line, config.max_syntax_length);
+                    // Re-split to get references into `line` with correct lifetimes.
+                    // SAFETY: slicing the string is safe because `truncate_str_short` always
+                    // returns a prefix of the input and only cuts at grapheme borders.
+                    let (with_syntax, plain) = line.split_at(line_syntax.len());
+                    // Note: splitting a line and only feeding one half to the highlighter may
+                    // result in wrong highlighting until it is reset the next hunk.
+                    //
+                    // Also, as lines are no longer newline terminated they might not be
+                    // highlighted correctly, and because of lifetimes inserting '\n' here is not
+                    // possible, also see `prepare()`.
+                    line_sections.push(
+                        highlighter
+                            .highlight_line(with_syntax, &config.syntax_set)
+                            .unwrap(),
+                    );
+
+                    if !plain.is_empty() {
+                        line_sections
+                            .last_mut()
+                            .unwrap()
+                            .push((config.null_syntect_style, plain));
+                    }
+                }
             }
         }
         _ => {
