@@ -1,6 +1,7 @@
 use std::io::{self, ErrorKind, IsTerminal, Read};
 
 use crate::cli;
+use crate::color::ColorMode;
 use crate::config;
 use crate::delta;
 use crate::env::DeltaEnv;
@@ -8,7 +9,7 @@ use crate::git_config;
 use crate::options::get::get_themes;
 use crate::utils::bat::output::{OutputType, PagingMode};
 
-pub fn show_themes(dark: bool, light: bool, computed_theme_is_light: bool) -> std::io::Result<()> {
+pub fn show_themes(dark: bool, light: bool, color_mode: ColorMode) -> std::io::Result<()> {
     use std::io::BufReader;
 
     use bytelines::ByteLines;
@@ -36,28 +37,30 @@ pub fn show_themes(dark: bool, light: bool, computed_theme_is_light: bool) -> st
 
     let git_config = git_config::GitConfig::try_create(&env);
     let opt = cli::Opt::from_iter_and_git_config(
-        env.clone(),
+        &env,
         &["delta", "--navigate", "--show-themes"],
         git_config,
     );
-    let mut output_type =
-        OutputType::from_mode(&env, PagingMode::Always, None, &config::Config::from(opt)).unwrap();
+    let mut output_type = OutputType::from_mode(
+        &env,
+        PagingMode::Always,
+        None,
+        &config::Config::from(opt).into(),
+    )
+    .unwrap();
     let title_style = ansi_term::Style::new().bold();
     let writer = output_type.handle().unwrap();
 
     for theme in &themes {
         let git_config = git_config::GitConfig::try_create(&env);
-        let opt = cli::Opt::from_iter_and_git_config(
-            env.clone(),
-            &["delta", "--features", theme],
-            git_config,
-        );
+        let opt =
+            cli::Opt::from_iter_and_git_config(&env, &["delta", "--features", theme], git_config);
         let is_dark_theme = opt.dark;
         let is_light_theme = opt.light;
         let config = config::Config::from(opt);
 
-        if (!computed_theme_is_light && is_dark_theme)
-            || (computed_theme_is_light && is_light_theme)
+        if (color_mode == ColorMode::Dark && is_dark_theme)
+            || (color_mode == ColorMode::Light && is_light_theme)
             || (dark && light)
         {
             writeln!(writer, "\n\nTheme: {}\n", title_style.paint(theme))?;
